from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="Bitset")


@attr.s(auto_attribs=True)
class Bitset:
    """
    Attributes:
        keys (List[str]):
        values (List[bool]):
    """

    keys: List[str]
    values: List[bool]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        keys = self.keys

        values = self.values

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "keys": keys,
                "values": values,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        keys = cast(List[str], d.pop("keys"))

        values = cast(List[bool], d.pop("values"))

        bitset = cls(
            keys=keys,
            values=values,
        )

        bitset.additional_properties = d
        return bitset

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import CustomEventSeverity, TeleopRequestData, StatefulEventMetadata, PortForwardingSessionRecord, TaskSummaryType, CustomEventTags, DatapointEventType, SystemEvent, DeviceOfflineEventMetadata, TeleopSessionRecordStreamType, DatapointEventMetadata, TaskSummary, EventSort, TaskSummaryTags, SystemEventType, StatefulEventTags, InterventionRequestInterventionType, DeviceOfflineEventTags, EventExportSheetResult, TeleopSessionRecordTags, CommentTags, CommentStreamType, CommandDeliveryEvent, PortForwardingSessionRecordMetadata, CommandDeliveryEventMetadata, PortForwardingSessionRecordType, LabelingRequestData, EventSortOrder, CommandDeliveryEventType, EventSortColumn, CommentMetadata, TeleopSessionRecordType, TeleopSessionRecord, EventSeekQueryEventTypesItem, DeviceOnlineEventTags, EventCounts, EventFilterEventTypesItem, DatapointEventStreamType, SystemEventTags, AnnotationType, AnnotationStreamType, DatapointEventSeverity, SystemEventMetadata, CustomEvent, AnnotationTags, LabeledPolygon, EventFilterTypesItem, CustomEventStreamType, TaskSummaryReport, TaskSummaryStreamType, EventFilter, CommandDeliveryEventTags, DeviceOfflineEventSeverity, StatefulEvent, EventHistogram, CommentType, EventListResponse, CustomEventType, InterventionRequestSeverity, DeviceOnlineEventSeverity, Comment, DeviceOnlineEventStreamType, EventQueryTypesItem, DatapointEvent, EventSeekQueryDirection, DeviceOfflineEvent, CustomEventMetadata, Annotation, DeviceOnlineEvent, EventSeekQuery, SelectionRequestData, EventQueryEventTypesItem, DeviceOnlineEventType, PhysicalRequestData, StatefulEventSeverity, PortForwardingSessionRecordTags, EventTriggerCommand, EventHistogramEntry, EventSeekQueryTypesItem, StatefulEventType, Label, InterventionRequestMetadata, EventExportSheetRequest, EventQuery, TeleopSessionRecordMetadata, DeviceOfflineEventType, StatefulEventStreamType, DeviceOnlineEventMetadata, SystemEventStreamType, CommandDeliveryEventStreamType, TaskSummaryMetadata, InterventionRequestStreamType, EventSeekQuerySeveritiesItem, DatapointEventTags, PortForwardingSessionRecordStreamType, InterventionRequestTags, InterventionRequest, InterventionRequestType, AnnotationMetadata, EventFilterSeveritiesItem, EventQuerySeveritiesItem, DeviceOfflineEventStreamType
from formant.sdk.cloud.v2.formant_admin_api_client.api.event import event_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.partial_user_language import PartialUserLanguage
from ..models.partial_user_region import PartialUserRegion
from ..models.partial_user_sms_opt_in_status import PartialUserSmsOptInStatus
from ..models.partial_user_units import PartialUserUnits
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.partial_user_tags import PartialUserTags
    from ..models.user_scope import UserScope


T = TypeVar("T", bound="PartialUser")


@attr.s(auto_attribs=True)
class PartialUser:
    """
    Attributes:
        organization_id (Union[Unset, str]): ID of the organization to which this user belongs.
        account_id (Union[Unset, None, str]): ID of the account to which this user belongs.
        role_id (Union[Unset, None, str]): ID of this user's role.
        email (Union[Unset, str]): Email address of this user.
        external_user_id (Union[Unset, None, str]): Ignore this - for internal use only.
        first_name (Union[Unset, str]): First name of this user.
        last_name (Union[Unset, str]): Last name of this user.
        scope (Union[Unset, None, UserScope]):
        team_id (Union[Unset, None, str]): ID of the team to which this user belongs.
        phone_number (Union[Unset, str]): User's phone number (e.g., +14155550100).
        enabled (Union[Unset, bool]): Enable or disable this user. Set this to False to delete this user.
        is_organization_owner (Union[Unset, bool]): Should this user have organization owner privileges?
        terms_accepted (Union[Unset, str]): Has this user accepted Formant terms and conditions?
        last_logged_in (Union[Unset, datetime.datetime]):
        password_hash (Union[Unset, None, str]):
        is_single_sign_on (Union[Unset, bool]):
        sms_opt_in_status (Union[Unset, None, PartialUserSmsOptInStatus]):
        region (Union[Unset, PartialUserRegion]):
        is_service_account (Union[Unset, bool]):
        allow_custom_email_configuration (Union[Unset, bool]):
        job_title (Union[Unset, str]):
        language (Union[Unset, PartialUserLanguage]):
        units (Union[Unset, PartialUserUnits]):
        timezone (Union[Unset, str]):
        title (Union[Unset, None, str]): Title for the user.
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, PartialUserTags]):
    """

    organization_id: Union[Unset, str] = UNSET
    account_id: Union[Unset, None, str] = UNSET
    role_id: Union[Unset, None, str] = UNSET
    email: Union[Unset, str] = UNSET
    external_user_id: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, str] = UNSET
    last_name: Union[Unset, str] = UNSET
    scope: Union[Unset, None, "UserScope"] = UNSET
    team_id: Union[Unset, None, str] = UNSET
    phone_number: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    is_organization_owner: Union[Unset, bool] = UNSET
    terms_accepted: Union[Unset, str] = UNSET
    last_logged_in: Union[Unset, datetime.datetime] = UNSET
    password_hash: Union[Unset, None, str] = UNSET
    is_single_sign_on: Union[Unset, bool] = UNSET
    sms_opt_in_status: Union[Unset, None, PartialUserSmsOptInStatus] = UNSET
    region: Union[Unset, PartialUserRegion] = UNSET
    is_service_account: Union[Unset, bool] = UNSET
    allow_custom_email_configuration: Union[Unset, bool] = UNSET
    job_title: Union[Unset, str] = UNSET
    language: Union[Unset, PartialUserLanguage] = UNSET
    units: Union[Unset, PartialUserUnits] = UNSET
    timezone: Union[Unset, str] = UNSET
    title: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "PartialUserTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        organization_id = self.organization_id
        account_id = self.account_id
        role_id = self.role_id
        email = self.email
        external_user_id = self.external_user_id
        first_name = self.first_name
        last_name = self.last_name
        scope: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.scope, Unset):
            scope = self.scope.to_dict() if self.scope else None

        team_id = self.team_id
        phone_number = self.phone_number
        enabled = self.enabled
        is_organization_owner = self.is_organization_owner
        terms_accepted = self.terms_accepted
        last_logged_in: Union[Unset, str] = UNSET
        if not isinstance(self.last_logged_in, Unset):
            last_logged_in = self.last_logged_in.isoformat()

        password_hash = self.password_hash
        is_single_sign_on = self.is_single_sign_on
        sms_opt_in_status: Union[Unset, None, str] = UNSET
        if not isinstance(self.sms_opt_in_status, Unset):
            sms_opt_in_status = self.sms_opt_in_status.value if self.sms_opt_in_status else None

        region: Union[Unset, str] = UNSET
        if not isinstance(self.region, Unset):
            region = self.region.value

        is_service_account = self.is_service_account
        allow_custom_email_configuration = self.allow_custom_email_configuration
        job_title = self.job_title
        language: Union[Unset, str] = UNSET
        if not isinstance(self.language, Unset):
            language = self.language.value

        units: Union[Unset, str] = UNSET
        if not isinstance(self.units, Unset):
            units = self.units.value

        timezone = self.timezone
        title = self.title
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if account_id is not UNSET:
            field_dict["accountId"] = account_id
        if role_id is not UNSET:
            field_dict["roleId"] = role_id
        if email is not UNSET:
            field_dict["email"] = email
        if external_user_id is not UNSET:
            field_dict["externalUserId"] = external_user_id
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if scope is not UNSET:
            field_dict["scope"] = scope
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if phone_number is not UNSET:
            field_dict["phoneNumber"] = phone_number
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if is_organization_owner is not UNSET:
            field_dict["isOrganizationOwner"] = is_organization_owner
        if terms_accepted is not UNSET:
            field_dict["termsAccepted"] = terms_accepted
        if last_logged_in is not UNSET:
            field_dict["lastLoggedIn"] = last_logged_in
        if password_hash is not UNSET:
            field_dict["passwordHash"] = password_hash
        if is_single_sign_on is not UNSET:
            field_dict["isSingleSignOn"] = is_single_sign_on
        if sms_opt_in_status is not UNSET:
            field_dict["smsOptInStatus"] = sms_opt_in_status
        if region is not UNSET:
            field_dict["region"] = region
        if is_service_account is not UNSET:
            field_dict["isServiceAccount"] = is_service_account
        if allow_custom_email_configuration is not UNSET:
            field_dict["allowCustomEmailConfiguration"] = allow_custom_email_configuration
        if job_title is not UNSET:
            field_dict["jobTitle"] = job_title
        if language is not UNSET:
            field_dict["language"] = language
        if units is not UNSET:
            field_dict["units"] = units
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if title is not UNSET:
            field_dict["title"] = title
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.partial_user_tags import PartialUserTags
        from ..models.user_scope import UserScope

        d = src_dict.copy()
        organization_id = d.pop("organizationId", UNSET)

        account_id = d.pop("accountId", UNSET)

        role_id = d.pop("roleId", UNSET)

        email = d.pop("email", UNSET)

        external_user_id = d.pop("externalUserId", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        _scope = d.pop("scope", UNSET)
        scope: Union[Unset, None, UserScope]
        if _scope is None:
            scope = None
        elif isinstance(_scope, Unset):
            scope = UNSET
        else:
            scope = UserScope.from_dict(_scope)

        team_id = d.pop("teamId", UNSET)

        phone_number = d.pop("phoneNumber", UNSET)

        enabled = d.pop("enabled", UNSET)

        is_organization_owner = d.pop("isOrganizationOwner", UNSET)

        terms_accepted = d.pop("termsAccepted", UNSET)

        _last_logged_in = d.pop("lastLoggedIn", UNSET)
        last_logged_in: Union[Unset, datetime.datetime]
        if isinstance(_last_logged_in, Unset):
            last_logged_in = UNSET
        else:
            last_logged_in = isoparse(_last_logged_in)

        password_hash = d.pop("passwordHash", UNSET)

        is_single_sign_on = d.pop("isSingleSignOn", UNSET)

        _sms_opt_in_status = d.pop("smsOptInStatus", UNSET)
        sms_opt_in_status: Union[Unset, None, PartialUserSmsOptInStatus]
        if _sms_opt_in_status is None:
            sms_opt_in_status = None
        elif isinstance(_sms_opt_in_status, Unset):
            sms_opt_in_status = UNSET
        else:
            sms_opt_in_status = PartialUserSmsOptInStatus(_sms_opt_in_status)

        _region = d.pop("region", UNSET)
        region: Union[Unset, PartialUserRegion]
        if isinstance(_region, Unset):
            region = UNSET
        else:
            region = PartialUserRegion(_region)

        is_service_account = d.pop("isServiceAccount", UNSET)

        allow_custom_email_configuration = d.pop("allowCustomEmailConfiguration", UNSET)

        job_title = d.pop("jobTitle", UNSET)

        _language = d.pop("language", UNSET)
        language: Union[Unset, PartialUserLanguage]
        if isinstance(_language, Unset):
            language = UNSET
        else:
            language = PartialUserLanguage(_language)

        _units = d.pop("units", UNSET)
        units: Union[Unset, PartialUserUnits]
        if isinstance(_units, Unset):
            units = UNSET
        else:
            units = PartialUserUnits(_units)

        timezone = d.pop("timezone", UNSET)

        title = d.pop("title", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, PartialUserTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = PartialUserTags.from_dict(_tags)

        partial_user = cls(
            organization_id=organization_id,
            account_id=account_id,
            role_id=role_id,
            email=email,
            external_user_id=external_user_id,
            first_name=first_name,
            last_name=last_name,
            scope=scope,
            team_id=team_id,
            phone_number=phone_number,
            enabled=enabled,
            is_organization_owner=is_organization_owner,
            terms_accepted=terms_accepted,
            last_logged_in=last_logged_in,
            password_hash=password_hash,
            is_single_sign_on=is_single_sign_on,
            sms_opt_in_status=sms_opt_in_status,
            region=region,
            is_service_account=is_service_account,
            allow_custom_email_configuration=allow_custom_email_configuration,
            job_title=job_title,
            language=language,
            units=units,
            timezone=timezone,
            title=title,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        partial_user.additional_properties = d
        return partial_user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import SheetParameters, AnnotationFieldValuesRequest, AnnotationField, ScopeFilter, UserParametersRolesItem, AnnotationTemplateTags, AnnotationFieldValuesResponse, AnnotationTemplateListResponse, AnnotationFieldValueTags, AnnotationType, AnnotationStreamType, Annotation, AnnotationTemplate, AnnotationFieldValuesRequestTags, AnnotationFieldType, AnnotationFieldValue, AnnotationMetadata, TagParameters, AnnotationTags, UserParameters, ScopeFilterTypesItem
from formant.sdk.cloud.v2.formant_admin_api_client.api.annotation import annotation_controller_post
from formant.sdk.cloud.v2.formant_admin_api_client.api.annotation_template import annotation_template_controller_list, annotation_template_controller_get_one
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="DeviceConfigurationTemplateTags")


@attr.s(auto_attribs=True)
class DeviceConfigurationTemplateTags:
    """ """

    additional_properties: Dict[str, str] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        device_configuration_template_tags = cls()

        device_configuration_template_tags.additional_properties = d
        return device_configuration_template_tags

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> str:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: str) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.http_integration_basic_auth_type import HttpIntegrationBasicAuthType

T = TypeVar("T", bound="HttpIntegrationBasicAuth")


@attr.s(auto_attribs=True)
class HttpIntegrationBasicAuth:
    """
    Attributes:
        type (HttpIntegrationBasicAuthType):
        username (str):
        password (str):
    """

    type: HttpIntegrationBasicAuthType
    username: str
    password: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        username = self.username
        password = self.password

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "username": username,
                "password": password,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = HttpIntegrationBasicAuthType(d.pop("type"))

        username = d.pop("username")

        password = d.pop("password")

        http_integration_basic_auth = cls(
            type=type,
            username=username,
            password=password,
        )

        http_integration_basic_auth.additional_properties = d
        return http_integration_basic_auth

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.partial_annotation_stream_type import PartialAnnotationStreamType
from ..models.partial_annotation_type import PartialAnnotationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.partial_annotation_metadata import PartialAnnotationMetadata
    from ..models.partial_annotation_tags import PartialAnnotationTags


T = TypeVar("T", bound="PartialAnnotation")


@attr.s(auto_attribs=True)
class PartialAnnotation:
    """
    Attributes:
        type (Union[Unset, PartialAnnotationType]):
        edited_at (Union[Unset, None, datetime.datetime]):
        user_id (Union[Unset, str]):
        annotation_template_id (Union[Unset, None, str]): If you want to create this annotation from an existing
            annotation template, enter its ID here.
        tagged_users (Union[Unset, Any]):
        published_to (Union[Unset, Any]):
        note (Union[Unset, None, str]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, PartialAnnotationTags]):
        organization_id (Union[Unset, str]): ID of the organization in which to create this new event.
        time (Union[Unset, datetime.datetime]): Start time of the data range relevant to this event.
        end_time (Union[Unset, None, datetime.datetime]): End time of the data range relevant to this event.
        parent_id (Union[Unset, str]): If this custom event is related to another event, you can add the ID of that
            parent event here.
        metadata (Union[Unset, PartialAnnotationMetadata]): User-customizable metadata associated with this event in
            key-value pair format.
        message (Union[Unset, str]): Message of this event. Must match the `message` field in the Formant UI when
            configuring a custom event.
        viewed (Union[Unset, bool]):
        device_id (Union[Unset, None, str]): ID of the device relevant to this event.
        stream_name (Union[Unset, None, str]):
        stream_type (Union[Unset, None, PartialAnnotationStreamType]):
        event_trigger_id (Union[Unset, None, str]):
        notification_enabled (Union[Unset, bool]): Set this to `true` to enable notifications for this event.
    """

    type: Union[Unset, PartialAnnotationType] = UNSET
    edited_at: Union[Unset, None, datetime.datetime] = UNSET
    user_id: Union[Unset, str] = UNSET
    annotation_template_id: Union[Unset, None, str] = UNSET
    tagged_users: Union[Unset, Any] = UNSET
    published_to: Union[Unset, Any] = UNSET
    note: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "PartialAnnotationTags"] = UNSET
    organization_id: Union[Unset, str] = UNSET
    time: Union[Unset, datetime.datetime] = UNSET
    end_time: Union[Unset, None, datetime.datetime] = UNSET
    parent_id: Union[Unset, str] = UNSET
    metadata: Union[Unset, "PartialAnnotationMetadata"] = UNSET
    message: Union[Unset, str] = UNSET
    viewed: Union[Unset, bool] = UNSET
    device_id: Union[Unset, None, str] = UNSET
    stream_name: Union[Unset, None, str] = UNSET
    stream_type: Union[Unset, None, PartialAnnotationStreamType] = UNSET
    event_trigger_id: Union[Unset, None, str] = UNSET
    notification_enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        edited_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.edited_at, Unset):
            edited_at = self.edited_at.isoformat() if self.edited_at else None

        user_id = self.user_id
        annotation_template_id = self.annotation_template_id
        tagged_users = self.tagged_users
        published_to = self.published_to
        note = self.note
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        organization_id = self.organization_id
        time: Union[Unset, str] = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.isoformat()

        end_time: Union[Unset, None, str] = UNSET
        if not isinstance(self.end_time, Unset):
            end_time = self.end_time.isoformat() if self.end_time else None

        parent_id = self.parent_id
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        message = self.message
        viewed = self.viewed
        device_id = self.device_id
        stream_name = self.stream_name
        stream_type: Union[Unset, None, str] = UNSET
        if not isinstance(self.stream_type, Unset):
            stream_type = self.stream_type.value if self.stream_type else None

        event_trigger_id = self.event_trigger_id
        notification_enabled = self.notification_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if edited_at is not UNSET:
            field_dict["editedAt"] = edited_at
        if user_id is not UNSET:
            field_dict["userId"] = user_id
        if annotation_template_id is not UNSET:
            field_dict["annotationTemplateId"] = annotation_template_id
        if tagged_users is not UNSET:
            field_dict["taggedUsers"] = tagged_users
        if published_to is not UNSET:
            field_dict["publishedTo"] = published_to
        if note is not UNSET:
            field_dict["note"] = note
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if time is not UNSET:
            field_dict["time"] = time
        if end_time is not UNSET:
            field_dict["endTime"] = end_time
        if parent_id is not UNSET:
            field_dict["parentId"] = parent_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if message is not UNSET:
            field_dict["message"] = message
        if viewed is not UNSET:
            field_dict["viewed"] = viewed
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if stream_name is not UNSET:
            field_dict["streamName"] = stream_name
        if stream_type is not UNSET:
            field_dict["streamType"] = stream_type
        if event_trigger_id is not UNSET:
            field_dict["eventTriggerId"] = event_trigger_id
        if notification_enabled is not UNSET:
            field_dict["notificationEnabled"] = notification_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.partial_annotation_metadata import PartialAnnotationMetadata
        from ..models.partial_annotation_tags import PartialAnnotationTags

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PartialAnnotationType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PartialAnnotationType(_type)

        _edited_at = d.pop("editedAt", UNSET)
        edited_at: Union[Unset, None, datetime.datetime]
        if _edited_at is None:
            edited_at = None
        elif isinstance(_edited_at, Unset):
            edited_at = UNSET
        else:
            edited_at = isoparse(_edited_at)

        user_id = d.pop("userId", UNSET)

        annotation_template_id = d.pop("annotationTemplateId", UNSET)

        tagged_users = d.pop("taggedUsers", UNSET)

        published_to = d.pop("publishedTo", UNSET)

        note = d.pop("note", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, PartialAnnotationTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = PartialAnnotationTags.from_dict(_tags)

        organization_id = d.pop("organizationId", UNSET)

        _time = d.pop("time", UNSET)
        time: Union[Unset, datetime.datetime]
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = isoparse(_time)

        _end_time = d.pop("endTime", UNSET)
        end_time: Union[Unset, None, datetime.datetime]
        if _end_time is None:
            end_time = None
        elif isinstance(_end_time, Unset):
            end_time = UNSET
        else:
            end_time = isoparse(_end_time)

        parent_id = d.pop("parentId", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, PartialAnnotationMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = PartialAnnotationMetadata.from_dict(_metadata)

        message = d.pop("message", UNSET)

        viewed = d.pop("viewed", UNSET)

        device_id = d.pop("deviceId", UNSET)

        stream_name = d.pop("streamName", UNSET)

        _stream_type = d.pop("streamType", UNSET)
        stream_type: Union[Unset, None, PartialAnnotationStreamType]
        if _stream_type is None:
            stream_type = None
        elif isinstance(_stream_type, Unset):
            stream_type = UNSET
        else:
            stream_type = PartialAnnotationStreamType(_stream_type)

        event_trigger_id = d.pop("eventTriggerId", UNSET)

        notification_enabled = d.pop("notificationEnabled", UNSET)

        partial_annotation = cls(
            type=type,
            edited_at=edited_at,
            user_id=user_id,
            annotation_template_id=annotation_template_id,
            tagged_users=tagged_users,
            published_to=published_to,
            note=note,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
            organization_id=organization_id,
            time=time,
            end_time=end_time,
            parent_id=parent_id,
            metadata=metadata,
            message=message,
            viewed=viewed,
            device_id=device_id,
            stream_name=stream_name,
            stream_type=stream_type,
            event_trigger_id=event_trigger_id,
            notification_enabled=notification_enabled,
        )

        partial_annotation.additional_properties = d
        return partial_annotation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_query_api_client.models import StreamDataListResponse, QueryAggregate, StreamAggregateData, BoundingBox, StreamAggregateDataTags, QueryTypesItem, Health,HealthStatus, Twist, Bitset, Localization, Path, PointCloud, Health, Quaternion, Goal, Odometry, StreamDataPointsItem, VideoMimeType, StreamDataTags, Video, Video,VideoMimeType, Image, Vector3, NumericSetEntry, StreamData, Query, Map, TransformNode, ImageAnnotation, Battery, File, StreamAggregateDataType, StreamDataType, Location, HealthStatus, Transform
from formant.sdk.cloud.v2.formant_query_api_client.api.query import query_controller_query
from formant.sdk.cloud.v2.formant_query_api_client.types import *



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.webhook import Webhook


T = TypeVar("T", bound="WebhooksInfo")


@attr.s(auto_attribs=True)
class WebhooksInfo:
    """
    Attributes:
        webhooks (List['Webhook']):
    """

    webhooks: List["Webhook"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        webhooks = []
        for webhooks_item_data in self.webhooks:
            webhooks_item = webhooks_item_data.to_dict()

            webhooks.append(webhooks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "webhooks": webhooks,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.webhook import Webhook

        d = src_dict.copy()
        webhooks = []
        _webhooks = d.pop("webhooks")
        for webhooks_item_data in _webhooks:
            webhooks_item = Webhook.from_dict(webhooks_item_data)

            webhooks.append(webhooks_item)

        webhooks_info = cls(
            webhooks=webhooks,
        )

        webhooks_info.additional_properties = d
        return webhooks_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict

import httpx

from ...client import AuthenticatedClient
from ...models.suggestion_request import SuggestionRequest
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: SuggestionRequest,
) -> Dict[str, Any]:
    url = "{}/suggestion/structured".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SuggestionRequest,
) -> Response[Any]:
    """Create structured suggestion

     Create a structured suggestions
    Resource: commands
    Authorized roles: viewer

    Args:
        json_body (SuggestionRequest):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SuggestionRequest,
) -> Response[Any]:
    """Create structured suggestion

     Create a structured suggestions
    Resource: commands
    Authorized roles: viewer

    Args:
        json_body (SuggestionRequest):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.updated_configuration_response import UpdatedConfigurationResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    *,
    client: AuthenticatedClient,
    reported_configuration_version: Union[Unset, None, int] = UNSET,
    agent_wall_clock_timestamp: Union[Unset, None, int] = UNSET,
    app_version: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/devices/{id}/updated-configuration".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(app_version, Unset):
        headers["app-version"] = app_version

    params: Dict[str, Any] = {}
    params["reportedConfigurationVersion"] = reported_configuration_version

    params["agentWallClockTimestamp"] = agent_wall_clock_timestamp

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[UpdatedConfigurationResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UpdatedConfigurationResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[UpdatedConfigurationResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    reported_configuration_version: Union[Unset, None, int] = UNSET,
    agent_wall_clock_timestamp: Union[Unset, None, int] = UNSET,
    app_version: Union[Unset, str] = UNSET,
) -> Response[UpdatedConfigurationResponse]:
    """Get updated configuration

     Check for updated device configuration
    Resource: devices
    Authorized roles: device

    Args:
        id (str):
        reported_configuration_version (Union[Unset, None, int]):
        agent_wall_clock_timestamp (Union[Unset, None, int]):
        app_version (Union[Unset, str]):

    Returns:
        Response[UpdatedConfigurationResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        reported_configuration_version=reported_configuration_version,
        agent_wall_clock_timestamp=agent_wall_clock_timestamp,
        app_version=app_version,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    reported_configuration_version: Union[Unset, None, int] = UNSET,
    agent_wall_clock_timestamp: Union[Unset, None, int] = UNSET,
    app_version: Union[Unset, str] = UNSET,
) -> Optional[UpdatedConfigurationResponse]:
    """Get updated configuration

     Check for updated device configuration
    Resource: devices
    Authorized roles: device

    Args:
        id (str):
        reported_configuration_version (Union[Unset, None, int]):
        agent_wall_clock_timestamp (Union[Unset, None, int]):
        app_version (Union[Unset, str]):

    Returns:
        Response[UpdatedConfigurationResponse]
    """

    return sync_detailed(
        id=id,
        client=client,
        reported_configuration_version=reported_configuration_version,
        agent_wall_clock_timestamp=agent_wall_clock_timestamp,
        app_version=app_version,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    reported_configuration_version: Union[Unset, None, int] = UNSET,
    agent_wall_clock_timestamp: Union[Unset, None, int] = UNSET,
    app_version: Union[Unset, str] = UNSET,
) -> Response[UpdatedConfigurationResponse]:
    """Get updated configuration

     Check for updated device configuration
    Resource: devices
    Authorized roles: device

    Args:
        id (str):
        reported_configuration_version (Union[Unset, None, int]):
        agent_wall_clock_timestamp (Union[Unset, None, int]):
        app_version (Union[Unset, str]):

    Returns:
        Response[UpdatedConfigurationResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        reported_configuration_version=reported_configuration_version,
        agent_wall_clock_timestamp=agent_wall_clock_timestamp,
        app_version=app_version,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    reported_configuration_version: Union[Unset, None, int] = UNSET,
    agent_wall_clock_timestamp: Union[Unset, None, int] = UNSET,
    app_version: Union[Unset, str] = UNSET,
) -> Optional[UpdatedConfigurationResponse]:
    """Get updated configuration

     Check for updated device configuration
    Resource: devices
    Authorized roles: device

    Args:
        id (str):
        reported_configuration_version (Union[Unset, None, int]):
        agent_wall_clock_timestamp (Union[Unset, None, int]):
        app_version (Union[Unset, str]):

    Returns:
        Response[UpdatedConfigurationResponse]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            reported_configuration_version=reported_configuration_version,
            agent_wall_clock_timestamp=agent_wall_clock_timestamp,
            app_version=app_version,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.event_list_response import EventListResponse
from ...models.event_query import EventQuery
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: EventQuery,
    app_id: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/events/query".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(app_id, Unset):
        headers["app-id"] = app_id

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[EventListResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = EventListResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[EventListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: EventQuery,
    app_id: Union[Unset, str] = UNSET,
) -> Response[EventListResponse]:
    """Query

     Query events
    Resource: events
    Authorized roles: viewer

    Args:
        app_id (Union[Unset, str]):
        json_body (EventQuery):

    Returns:
        Response[EventListResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        app_id=app_id,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: EventQuery,
    app_id: Union[Unset, str] = UNSET,
) -> Optional[EventListResponse]:
    """Query

     Query events
    Resource: events
    Authorized roles: viewer

    Args:
        app_id (Union[Unset, str]):
        json_body (EventQuery):

    Returns:
        Response[EventListResponse]
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        app_id=app_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: EventQuery,
    app_id: Union[Unset, str] = UNSET,
) -> Response[EventListResponse]:
    """Query

     Query events
    Resource: events
    Authorized roles: viewer

    Args:
        app_id (Union[Unset, str]):
        json_body (EventQuery):

    Returns:
        Response[EventListResponse]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        app_id=app_id,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: EventQuery,
    app_id: Union[Unset, str] = UNSET,
) -> Optional[EventListResponse]:
    """Query

     Query events
    Resource: events
    Authorized roles: viewer

    Args:
        app_id (Union[Unset, str]):
        json_body (EventQuery):

    Returns:
        Response[EventListResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            app_id=app_id,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CheckSsoRequest")


@attr.s(auto_attribs=True)
class CheckSsoRequest:
    """
    Attributes:
        email (str):
        allow_user_auto_creation (Union[Unset, bool]):
    """

    email: str
    allow_user_auto_creation: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        email = self.email
        allow_user_auto_creation = self.allow_user_auto_creation

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "email": email,
            }
        )
        if allow_user_auto_creation is not UNSET:
            field_dict["allowUserAutoCreation"] = allow_user_auto_creation

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        email = d.pop("email")

        allow_user_auto_creation = d.pop("allowUserAutoCreation", UNSET)

        check_sso_request = cls(
            email=email,
            allow_user_auto_creation=allow_user_auto_creation,
        )

        check_sso_request.additional_properties = d
        return check_sso_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import PartialViewTags, LocationViewport, Camera, ImageViewConfigurationMode, PartialViewLayoutType, ViewTags, BitsetViewConfiguration, PointCloudViewConfiguration, ViewConfiguration, View, ViewConfigurationType, Vector3, ViewLayoutType, PartialView, LocationViewConfigurationBasemap, TransformTreeViewConfiguration, LocalizationViewConfiguration, Filter, LocationViewConfiguration, NumericViewConfiguration, Location, ViewListResponse, ImageViewConfiguration
from formant.sdk.cloud.v2.formant_admin_api_client.api.view import view_controller_get_one, view_controller_get_all, view_controller_patch
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DeviceDiskConfiguration")


@attr.s(auto_attribs=True)
class DeviceDiskConfiguration:
    """
    Attributes:
        buffer_size (Union[Unset, int]):
        on_demand_buffer_size (Union[Unset, int]):
    """

    buffer_size: Union[Unset, int] = UNSET
    on_demand_buffer_size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        buffer_size = self.buffer_size
        on_demand_buffer_size = self.on_demand_buffer_size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if buffer_size is not UNSET:
            field_dict["bufferSize"] = buffer_size
        if on_demand_buffer_size is not UNSET:
            field_dict["onDemandBufferSize"] = on_demand_buffer_size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        buffer_size = d.pop("bufferSize", UNSET)

        on_demand_buffer_size = d.pop("onDemandBufferSize", UNSET)

        device_disk_configuration = cls(
            buffer_size=buffer_size,
            on_demand_buffer_size=on_demand_buffer_size,
        )

        device_disk_configuration.additional_properties = d
        return device_disk_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NumericCondition")


@attr.s(auto_attribs=True)
class NumericCondition:
    """
    Attributes:
        label (str):
        threshold (Union[Unset, Any]):
    """

    label: str
    threshold: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        label = self.label
        threshold = self.threshold

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "label": label,
            }
        )
        if threshold is not UNSET:
            field_dict["threshold"] = threshold

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label")

        threshold = d.pop("threshold", UNSET)

        numeric_condition = cls(
            label=label,
            threshold=threshold,
        )

        numeric_condition.additional_properties = d
        return numeric_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import DeviceConfiguration, NetworkInfo, DeviceStreamHardwareConfigurationType, DeviceReportedConfigurationState, DeviceStreamFileTailConfigurationFileFormat, OsInfo, DeviceTeleopHardwareStreamConfigurationMode, DeviceFollower, OnDemandBufferBufferType, PartialDevice, ScopeFilter, DeviceStreamRosTransformTreeConfiguration, DeviceTeleopRosStreamConfiguration, KernelInfo, DeviceOfflineEventMetadata, DeviceBlobData, DeviceTeleopHardwareStreamConfigurationRtcStreamType, DeviceConfigurationTemplateListResponse, DeviceScopeTypesItem, DeviceStreamDirectoryWatchConfigurationType, DeviceTeleopCustomStreamConfigurationMode, OnDemandPresenceStreamItemGroup, DeviceTeleopRosStreamConfigurationTopicType, NodeInfo, DeviceOfflineEventTags, DeviceStreamCustomConfiguration, TeleopJoystickAxisConfiguration, DeviceCredentials, AdapterConfiguration, DeviceTelemetryConfiguration, DeviceDetails, OnDemandPresenceStreamItemGroupDatapointType, DeviceTeleopRosStreamConfigurationQuality, OnDemandState, DeviceStreamDirectoryWatchConfigurationFileType, DeviceStreamCustomConfigurationType, DeviceStreamConfiguration, TeleopViewConfiguration, DeviceTeleopCustomStreamConfigurationNumericControlVisualization, TeleopJoystickConfiguration, DeviceTeleopCustomStreamConfigurationRtcStreamType, AudioDevice, DeviceOnlineEventTags, DeviceTeleopHardwareStreamConfigurationHardwareType, DeviceStreamHardwareConfigurationHardwareType, PartialDeviceTags, DeviceProvisioning, RosTopic, DeviceStreamConfigurationTags, DeviceRosConfiguration, DeviceStreamRosTopicConfigurationType, DeviceTags, DeviceStreamFileTailConfiguration, ValidationConfiguration, DeviceProvisioningRequest, DeviceConfigurationTemplateTags, DeviceQueryType, DeviceTeleopRosStreamConfigurationNumericControlVisualization, DeviceApplicationConfigurationConfigurationMap, DeviceDiskConfiguration, RtcInfoRtcIceServerProtocol, OnDemandPresenceTimeRange, DeviceOfflineEventSeverity, DeviceType, OnvifDevice, DeviceRosState, DeviceTeleopConfiguration, DeviceOnlineEventSeverity, DeviceOnlineEventStreamType, DeviceDiagnosticsConfiguration, DeviceStreamHardwareConfiguration, DeviceOfflineEvent, DeviceDetailsType, DeviceStreamRosLocalizationConfigurationType, DeviceConfigurationDocument, DeviceOnlineEvent, DeviceDetailsTags, RtcInfo, Device, DeviceOnlineEventType, DeviceStateEnv, TeleopJoystickAxisConfigurationDimension, OnDemandStreamPresence, DevicePortForwardingConfiguration, DeviceStreamFileTailConfigurationType, TeleopJoystickConfigurationPosition, DeviceQuery, DeviceConfigurationTemplate, DeviceStreamDirectoryWatchConfiguration, Network, DeviceApplicationConfiguration, DeviceStreamRosTopicConfiguration, DeviceOfflineEventStreamType, DeviceConfigurationDocumentTags, CommandProgress, DeviceStreamTransformConfiguration, DeviceStreamHardwareConfigurationQuality, PartialDeviceType, HwInfo, DeviceStreamRosTransformTreeConfigurationType, DeviceState, DeviceOfflineEventType, DeviceTeleopRosStreamConfigurationMode, DeviceOnlineEventMetadata, DeviceListResponse, DeviceDetailsListResponse, DeviceScope, DeviceTeleopRosStreamConfigurationAudioCodec, OnDemandBuffer, DeviceTeleopCustomStreamConfiguration, DeviceTeleopCustomStreamConfigurationQuality, DeviceStreamRosLocalizationConfiguration, DeviceResourcesConfiguration, DeviceStreamConfigurationQuality, DeviceTeleopHardwareStreamConfiguration, VideoDevice, RtcInfoRtcIceTransportPoliciesItem, ScopeFilterTypesItem, DeviceTeleopHardwareStreamConfigurationQuality
from formant.sdk.cloud.v2.formant_admin_api_client.api.device import device_controller_query, device_controller_patch, device_controller_get_one, device_controller_get_configuration, device_controller_post_configuration
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from formant.sdk.cloud.v2.formant_admin_api_client.types import Optional




from formant.sdk.cloud.v2.formant_admin_api_client.models import CommandStreamType, CommandDeliveryEvent, FileInfo, CommandResponseStreamType, CommandDeliveryEventMetadata, CommandTags, CommandDeliveryEventTags, CommandTemplateTags, CommandDeliveryEventType, CommandProgress, ScopeFilter, CommandListResponse, CommandQuery, CommandParameter, CommandResponse, CommandParameterMeta, CommandDeliveryEventStreamType, CommandTemplateParameterMeta, CommandResponseTags, CommandTemplate, Command, CommandRequest, CommandRequestListResponse, ScopeFilterTypesItem, CommandTemplateListResponse
from formant.sdk.cloud.v2.formant_admin_api_client.api.command import command_controller_post, command_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
